package nest_benchmark

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"log"
	"net/http"
	"sync"

	"gitea.urkob.com/urko/crono"
)

type GraphqlQuery struct {
	Query         string      `json:"query"`
	OperationName string      `json:"operationName"`
	Variables     interface{} `json:"variables"`
}

type ByEmployeeNumberQueryVariables struct {
	EmployeeNumber float64 `json:"employeeNumber"`
}

func doRequest(wg *sync.WaitGroup, host string, employeeID int, errChan chan error) error {
	defer wg.Done()
	var err error
	query := GraphqlQuery{
		Query:         `query ByEmployeeNumber($employeeNumber: Float!) { byEmployeeNumber(employeeNumber: $employeeNumber) { EmployeeNumber Date WorkInformation { EmployeeNumber Date Shifts { Start End RoleCode } DaysOff { DayOff { DayOffTypeCode Note _ReferenceId } } } Baselines { Shifts { Start End RoleCode } BaselineType DaysOff { DayOff { DayOffTypeCode } } FullDayAbsences { FullDayAbsence { AbsenceTypeCode } } } CustomFields { CustomField { Name } } DataVersion FullDayAbsences { FullDayAbsence { _ReferenceId } } } } `,
		OperationName: "ByEmployeeNumber",
		Variables: ByEmployeeNumberQueryVariables{
			EmployeeNumber: float64(employeeID),
		},
	}

	reqBody, err := json.Marshal(query)
	if err != nil {
		err = fmt.Errorf("json.Marshal: %s", err)
		errChan <- err
		return err
	}

	client := &http.Client{}
	req, err := http.NewRequest("POST", host, bytes.NewReader(reqBody))

	if err != nil {
		err = fmt.Errorf("http.NewRequest: %s", err)
		errChan <- err
		return err
	}
	req.Header.Add("Content-Type", "application/json")

	res, err := client.Do(req)
	if err != nil {
		err = fmt.Errorf("client.Do: %s", err)
		errChan <- err
		return err
	}
	defer res.Body.Close()

	readedBody, err := io.ReadAll(res.Body)
	if err != nil {
		err = fmt.Errorf("ioutil.ReadAll: %s", err)
		errChan <- err
		return err
	}

	log.Println("readed", string(readedBody))

	return nil
}

func BenchmarkNoLog(host string, employeeNumber, totalRequests int) error {
	return benchmark(host, employeeNumber, totalRequests, false)
}

func Benchmark(host string, employeeNumber, totalRequests int) error {
	return benchmark(host, employeeNumber, totalRequests, true)
}

func benchmark(host string, employeeNumber int, totalRequests int, dolog bool) error {
	cr := crono.New()
	if dolog {
		defer cr.Table()
	}

	wg := &sync.WaitGroup{}
	errChan := make(chan error, 1)
	wg.Add(1)
	go func(errChan chan error) {
		defer wg.Done()
		defer func() {
			errChan <- nil
		}()
		for i := 0; i < totalRequests; i++ {
			wg.Add(1)
			go doRequest(wg, host, employeeNumber, errChan)
		}
	}(errChan)

	wg.Wait()

	if err := <-errChan; err != nil {
		return fmt.Errorf("Benchmark: %s", err)
	}
	if dolog {
		cr.MarkAndRestart(fmt.Sprintf("nest handle %d response", totalRequests))
	}
	return nil
}
